@@ -9,7 +9,7 @@ from tqdm import tqdm
 from contextlib import ExitStack
 from .short_read import Coverage
 from ._utils import junctions_from_cigar, splice_identical, is_same_gene, has_overlap, get_overlap, pairwise, \
-    cigar_string2tuples, rc, get_intersects, _find_splice_sites, _get_overlap  # , _get_exonic_region
+    cigar_string2tuples, rc, get_intersects, _find_splice_sites, _get_overlap, get_quantiles  # , _get_exonic_region
 from .gene import Gene
 from .decorators import experimental
 import logging
@@ -247,7 +247,8 @@ def add_sample_from_csv(self, coverage_csv_file, transcripts_file, transcript_id
     # todo: extend sample_table
     for sa in samples:
         sample_properties[sa].update({'name': sa, 'file': coverage_csv_file, 'nonchimeric_reads': cov_tab[sample_cov_cols[sa]].sum(), 'chimeric_reads': 0})
-        self.infos['sample_table'] = self.sample_table.append(sample_properties[sa], ignore_index=True)
+        # self.infos['sample_table'] = self.sample_table.append(sample_properties[sa], ignore_index=True)
+        self.infos['sample_table'] = pd.concat([self.sample_table, pd.DataFrame([sample_properties[sa]])])
 
     self.make_index()
     return id_map
@@ -419,8 +420,8 @@ def add_sample_from_bam(self, fn, sample_name=None, barcode_file=None, fuzzy_jun
                         starts[r[0]] = starts.get(r[0], 0) + cov
                         ends[r[1]] = ends.get(r[1], 0) + cov
                     # get the median TSS/PAS
-                    tr['exons'][0][0] = get_quantile(starts.items(), 0.5)
-                    tr['exons'][-1][1] = get_quantile(ends.items(), 0.5)
+                    tr['exons'][0][0] = get_quantiles(starts.items(), [0.5])[0]
+                    tr['exons'][-1][1] = get_quantiles(ends.items(), [0.5])[0]
                     cov = sum(tr_ranges.values())
                     s_name = tr.get('bc_group', sample_name)
                     tr['coverage'] = {s_name: cov}
@@ -450,9 +451,9 @@ def add_sample_from_bam(self, fn, sample_name=None, barcode_file=None, fuzzy_jun
         logger.info('ignored %s reads marked as unaligned', unmapped)
     if n_lowqual > 0:
         logger.info('ignored %s reads with mapping quality < %s', n_lowqual, min_mapqual)
-    
+
     # merge chimeric reads and assign gene names
-    
+
     n_chimeric = dict()
     # split up chimeric reads in actually chimeric and long introns (non_chimeric)
     # chimeric are grouped by breakpoint (bp->{readname->(chrom, strand, exons)})
@@ -460,13 +461,15 @@ def add_sample_from_bam(self, fn, sample_name=None, barcode_file=None, fuzzy_jun
     # sample_nc_reads[sa]
 
     n_chimeric = _add_chimeric(self, chim, chimeric_mincov, min_exonic_ref_coverage)
-    for nc_cov_dict, _ , _ in non_chimeric.values():
+    n_long_intron = {}
+    for nc_cov_dict, _, _ in non_chimeric.values():
         for sa, nc_cov in nc_cov_dict.items():
-            sample_nc_reads[sa] = sample_nc_reads.get(sa,0) + nc_cov
+            n_long_intron[sa] = n_long_intron.get(sa, 0) + nc_cov
+            sample_nc_reads[sa] = sample_nc_reads.get(sa, 0) + nc_cov
     chim_ignored = sum(len(chim) for chim in chimeric.values()) - sum(n_chimeric.values())
     if chim_ignored > 0:
         logger.info('ignoring %s chimeric alignments with less than %s reads', chim_ignored, chimeric_mincov)
-    chained_msg = '' if not sum(sample_nc_reads.values()) else f' (including  {sum(sample_nc_reads.values())} chained chimeric alignments)'
+    chained_msg = '' if not sum(n_long_intron.values()) else f' (including  {sum(n_long_intron.values())} chained chimeric alignments)'
     chimeric_msg = '' if sum(n_chimeric.values()) == 0 else f' and {sum(n_chimeric.values())} chimeric reads with coverage of at least {chimeric_mincov}'
     logger.info('imported %s nonchimeric reads%s%s.', sum(sample_nc_reads.values()), chained_msg, chimeric_msg)
 
@@ -475,7 +478,7 @@ def add_sample_from_bam(self, fn, sample_name=None, barcode_file=None, fuzzy_jun
         try:
             tss, pas = (exons[0][0], exons[-1][1]) if strand == '+' else (exons[-1][1], exons[0][0])
             tr = {'exons': exons, 'coverage': cov, 'TSS': {sa: {tss: c} for sa, c in cov.items()}, 'PAS': {sa: {pas: c} for sa, c in cov.items()},
-                'strand': strand, 'chr': chrom, 'long_intron_chimeric': introns}
+                  'strand': strand, 'chr': chrom, 'long_intron_chimeric': introns}
             if save_readnames:
                 tr['reads'] = {s_name: [readname]}
         except BaseException:
@@ -495,7 +498,8 @@ def add_sample_from_bam(self, fn, sample_name=None, barcode_file=None, fuzzy_jun
         kwargs['chimeric_reads'] = n_chimeric.get(s_name, 0)
         kwargs['nonchimeric_reads'] = sample_nc_reads.get(s_name, 0)
         kwargs['name'] = s_name
-        self.infos['sample_table'] = self.sample_table.append(kwargs, ignore_index=True)
+        # self.infos['sample_table'] = self.sample_table.append(kwargs, ignore_index=True)
+        self.infos['sample_table'] = pd.concat([self.sample_table, pd.DataFrame([kwargs])])
     self.make_index()
     return total_nc_reads_chr
 
@@ -547,17 +551,6 @@ def _add_chimeric(t, new_chimeric, min_cov, min_exonic_ref_coverage):
     return total
 
 
-def get_quantile(pos, percentile=.5):
-    '''provided a list of (positions,coverage) pairs, return the median position'''
-    total = sum(cov for _, cov in pos)
-    n = 0
-    for p, cov in sorted(pos, key=lambda x: x[0]):
-        n += cov
-        if n >= total * percentile:
-            return(p)
-    raise ValueError(f'cannot find {percentile} percentile of {pos}')
-
-
 def _breakpoints(chimeric):
     ''' gets chimeric aligment as a list and returns list of breakpoints.
         each breakpoint is a tuple of (chr1, strand1, pos1,  chr2,strand2,pos2)
@@ -619,7 +612,8 @@ def _check_chimeric(chimeric):
             chimeric_dict.setdefault(bpts, {})[readname] = new_chim
         else:
             assert len(new_chim[1]) == 1
-            non_chimeric[readname] = [new_chim[0], new_chim[1][0], tuple(merged_introns)]  # coverage, part(chrom,strand,exons,[aligned start, end]), and "long introns"
+            # coverage, part(chrom,strand,exons,[aligned start, end]), and "long introns"
+            non_chimeric[readname] = [new_chim[0], new_chim[1][0], tuple(merged_introns)]
     if skip:
         logger.warning('ignored %s chimeric alignments with only one part aligned to specified chromosomes.', skip)
     return chimeric_dict, non_chimeric
@@ -770,12 +764,12 @@ def _combine_transcripts(established, new_tr):
         ends = [v for sa in established['PAS'] for v in established['PAS'][sa].items()]
         if established['strand'] == '-':
             starts, ends = ends, starts
-        established['exons'][0][0] = get_quantile(starts, 0.5)
-        established['exons'][-1][1] = get_quantile(ends, 0.5)
+        established['exons'][0][0] = get_quantiles(starts, [0.5])[0]
+        established['exons'][-1][1] = get_quantiles(ends, [0.5])[0]
         if 'long_intron_chimeric' in new_tr:
-            new_introns=set(new_tr['long_intron_chimeric'])
-            established_introns=set(established.get('long_intron_chimeric', set()))
-            established['long_intron_chimeric']=tuple(new_introns.union(established_introns))
+            new_introns = set(new_tr['long_intron_chimeric'])
+            established_introns = set(established.get('long_intron_chimeric', set()))
+            established['long_intron_chimeric'] = tuple(new_introns.union(established_introns))
     except BaseException:
         logger.error('error when merging %s into %s', new_tr, established)
         raise
@@ -1223,10 +1217,10 @@ def aligned_part(cigartuples, is_reverse):
 def get_clipping(cigartuples, pos):
     if cigartuples[0][0] == 4:
         # clipping at the begining
-        return(pos, -cigartuples[0][1])
+        return (pos, -cigartuples[0][1])
     elif cigartuples[-1][0] == 4:
         # clipping at the end - get the reference position
-        return(pos + sum(c[1] for c in cigartuples[:-1] if c[0] in (0, 2, 3, 7, 8)), cigartuples[-1][1])  # MDN=X -> move forward on reference:
+        return (pos + sum(c[1] for c in cigartuples[:-1] if c[0] in (0, 2, 3, 7, 8)), cigartuples[-1][1])  # MDN=X -> move forward on reference:
     else:
         return None
 
@@ -1247,7 +1241,7 @@ def _set_alias(d, alias, required=True):
 
 
 # human readable output
-def gene_table(self, **filter_args):  # ideas: filter, extra_columns
+def gene_table(self, **filter_args):  # ideas: extra_columns
     '''Creates a gene summary table.
 
     Exports all genes within region to a table.
@@ -1257,7 +1251,7 @@ def gene_table(self, **filter_args):  # ideas: filter, extra_columns
     colnames = ['chr', 'start', 'end', 'strand', 'gene_id', 'gene_name', 'n_transcripts']
     rows = [(g.chrom, g.start, g.end, g.strand, g.id, g.name, g.n_transcripts) for g in self.iter_genes(**filter_args)]
     df = pd.DataFrame(rows, columns=colnames)
-    return(df)
+    return df
 
 
 def transcript_table(self,  samples=None,  groups=None, coverage=False, tpm=False, tpm_pseudocount=0, extra_columns=None,  **filter_args):
@@ -1345,7 +1339,7 @@ def transcript_table(self,  samples=None,  groups=None, coverage=False, tpm=Fals
                 df_list.append(((cov_gr+tpm_pseudocount)/total*1e6).add_suffix('_sum_tpm'))
         df = pd.concat(df_list, axis=1)
 
-    return(df)
+    return df
 
 
 @ experimental
@@ -1575,7 +1569,7 @@ def get_gff_chrom_dict(gff, chromosomes):
             if chromosomes is None or c in chromosomes:
                 chrom[c] = c
     gff.seek(0)
-    return(chrom)
+    return chrom
 
 
 class IntervalArray:
@@ -1604,7 +1598,7 @@ class IntervalArray:
             raise
 
     def __len__(self):
-        return(len(self.obj))
+        return len(self.obj)
 
     def __iter__(self):
         return (v for v in self.obj.values())
