@@ -6,10 +6,10 @@ import logging
 import numpy as np
 import pandas as pd
 import itertools
-from scipy.stats import chi2_contingency, fisher_exact
+from ._utils import has_overlap
 
 # from .decorators import deprecated, debug, experimental
-from ._utils import _filter_function, _corrected_log2OR
+from ._utils import _filter_function
 
 logger = logging.getLogger('isotools')
 
@@ -25,7 +25,7 @@ def proportion_test(x, n):
     p1 = [x[i] / n[i] for i in range(2)]
     p0 = (x[0] + x[1]) / (n[0] + n[1])
     z = abs(p1[0] - p1[1]) / np.sqrt(p0 * (1 - p0) * (1 / n[0] + 1 / n[1]))
-    return(2 * norm.sf(z)), (p1[0], 0, p1[1], 0, p0, 0)  # two sided alternative
+    return (2 * norm.sf(z)), (p1[0], 0, p1[1], 0, p0, 0)  # two sided alternative
 
 
 def binom_lr_test(x, n):
@@ -149,12 +149,13 @@ def _check_groups(transcriptome, groups, n_groups=2):
     if notfound:
         raise ValueError(f"Cannot find the following samples: {notfound}")
     assert all((gn1 not in gn2 for gn1, gn2 in itertools.permutations(groupnames, 2))), 'group names must not be contained in other group names'
-    sa_idx = {sa: idx[0] for sa, idx in transcriptome._get_sample_idx().items()}
+    sa_idx = {sa: idx for sa, idx in transcriptome._get_sample_idx().items()}
     grp_idx = [[sa_idx[sa] for sa in grp] for grp in groups]
     return groupnames, groups, grp_idx
 
 
-def altsplice_test(self, groups, min_total=100, min_alt_fraction=.1, min_n=10, min_sa=.51, test='auto', padj_method='fdr_bh', types=None, progress_bar=True):
+def altsplice_test(self, groups, min_total=100, min_alt_fraction=.1, min_n=10, min_sa=.51, test='auto', padj_method='fdr_bh',
+                   types=None, domains=None, progress_bar=True):
     '''Performs the alternative splicing event test.
 
     :param groups: Dict with groupnames as keys and lists of samplenames as values, defining the two groups for the test.
@@ -166,10 +167,21 @@ def altsplice_test(self, groups, min_total=100, min_alt_fraction=.1, min_n=10, m
     :param min_sa: The fraction of samples within each group that must be covered by at least min_n reads.
     :param test: The name of one of the implemented statistical tests ('betabinom_lr','binom_lr','proportions').
     :param padj_method: Specify the method for multiple testing correction.
-    :param types: Restrict the analysis on types of events. If ommited, all types are tested.'''
+    :param types: Restrict the analysis on types of events. If ommited, all types are tested.
+    :param domains: Protin domains overlapping the alternative splicing event get added to the result table.
+        Specify a list of sources, or a dictionary of sources and list of categories to be considdered.
+        By default, domains are not added.'''
 
+    if not domains:
+        domains = {}
+    elif isinstance(domains, str):
+        domains = {domains: None}
+    elif not isinstance(domains, dict):
+        domains = {k: None for k in domains}
+
+    noORF = (None, None, {'NMD': True})
     groupnames, groups, grp_idx = _check_groups(self, groups)
-    sidx = grp_idx[0] + grp_idx[1]
+    sidx = np.array(grp_idx[0] + grp_idx[1])
 
     if isinstance(test, str):
         if test == 'auto':
@@ -204,11 +216,11 @@ def altsplice_test(self, groups, min_total=100, min_alt_fraction=.1, min_n=10, m
                     known.setdefault(splice_type, set()).add((sg[nX].end, sg[nY].start))
         sg = g.segment_graph
         for setA, setB, nX, nY, splice_type in sg.find_splice_bubbles(types=types):
-
             junction_cov = g.coverage[:, setB].sum(1)
             total_cov = g.coverage[:, setA].sum(1) + junction_cov
             if total_cov[sidx].sum() < min_total:
                 continue
+
             alt_fraction = junction_cov[sidx].sum() / total_cov[sidx].sum()
             if alt_fraction < min_alt_fraction or alt_fraction > 1 - min_alt_fraction:
                 continue
@@ -227,12 +239,35 @@ def altsplice_test(self, groups, min_total=100, min_alt_fraction=.1, min_n=10, m
             else:
                 start, end = sg[nX].end, sg[nY].start
                 novel = (start, end) not in known.get(splice_type, set())
-            res.append(tuple(itertools.chain((g.name, g.id, g.chrom, g.strand, start, end, splice_type, novel, pval, setA, setB), params, params_other,
-                                             (val for lists in zip(x, n) for pair in zip(*lists) for val in pair))))
+            event_dom = {}
+            if domains:  # check for overlapping domains
+                for dom_source, dom_categories in domains.items():
+                    oltr = set()
+                    for trid in setA+setB:  # these are actually lists
+                        for dom in g.transcripts[trid].get('domain', {}).get(dom_source, []):
+                            if (dom_categories is None or
+                                    dom[1] in dom_categories) and \
+                                    has_overlap(dom[3], (start, end)):
+                                oltr.add(str(dom[0]))
+                    event_dom[dom_source] = (len(oltr), ','.join(oltr))
 
-    df = pd.DataFrame(res, columns=(['gene', 'gene_id', 'chrom', 'strand', 'start', 'end', 'splice_type', 'novel', 'pvalue', 'trA', 'trB'] +
-                                    [gn + part for gn in groupnames[:2] + ['total'] + groupnames[2:] for part in ['_PSI', '_disp']] +
-                                    [f'{sa}_{gn}_{w}' for gn, grp in zip(groupnames, groups) for sa in grp for w in ['in_cov', 'total_cov']]))
+            nmdA = sum(g.coverage[np.ix_(sidx, [trid])].sum(None)
+                       for trid in setA if g.transcripts[trid].get('ORF', noORF)[2]['NMD'])/g.coverage[np.ix_(sidx, setA)].sum(None)
+            nmdB = sum(g.coverage[np.ix_(sidx, [trid])].sum(None)
+                       for trid in setB if g.transcripts[trid].get('ORF', noORF)[2]['NMD'])/g.coverage[np.ix_(sidx, setB)].sum(None)
+            res.append(tuple(itertools.chain((g.name, g.id, g.chrom, g.strand, start, end, splice_type, novel, pval,
+                                             sorted(setA, key=lambda x: -g.coverage[np.ix_(sidx, [x])].sum(0)),
+                                             sorted(setB, key=lambda x: -g.coverage[np.ix_(sidx, [x])].sum(0)), nmdA, nmdB),
+                                             (v for dom_source in domains for v in event_dom[dom_source]),
+                                             params, params_other,
+                                             (val for lists in zip(x, n) for pair in zip(*lists) for val in pair))))
+    colnames = ['gene', 'gene_id', 'chrom', 'strand', 'start', 'end', 'splice_type', 'novel', 'pvalue', 'trA', 'trB', 'nmdA', 'nmdB']
+    if domains:
+        for dom_source in domains:
+            colnames += [f'n_{dom_source}_domains', f'{dom_source}_domains']
+    colnames += [gn + part for gn in groupnames[:2] + ['total'] + groupnames[2:] for part in ['_PSI', '_disp']]
+    colnames += [f'{sa}_{gn}_{w}' for gn, grp in zip(groupnames, groups) for sa in grp for w in ['in_cov', 'total_cov']]
+    df = pd.DataFrame(res, columns=colnames)
     try:
         mask = np.isfinite(df['pvalue'])
         padj = np.empty(mask.shape)
@@ -251,8 +286,6 @@ def die_test(self, groups, min_cov=25, n_isoforms=10, padj_method='fdr_bh', prog
     Syntax and parameters follow the original implementation in
     https://github.com/noush-joglekar/scisorseqr/blob/master/inst/RScript/IsoformTest.R
     :param groups: Dict with groupnames as keys and lists of samplenames as values, defining the two groups for the test.
-    If more then two groups are provided, test is performed between first two groups, but maximum likelihood parameters
-    (expected PSI and dispersion) will be computet for the other groups as well.
     :param min_cov: Minimal number of reads per group for each gene.
     :param n_isoforms: Number of isoforms to consider in the test for each gene. All additional least expressed isoforms get summarized.'''
 
@@ -738,76 +771,8 @@ def rarefaction(self, groups=None, fractions=20, min_coverage=2, tr_filter={}):
     return pd.DataFrame(curves, index=fractions), total
 
 
-def pairwise_event_test(e1, e2, coverage, test="chi2"):
-    '''
-    performs an independence test among the joint frequencies of two events
-
-    :param e1: event obtained from .find_splice_bubbles()
-    :param e2: event obtained from .find_splice_bubbles()
-    :param transcriptome: the Transcriptome object on which the events were computed
-    :param coverage: list of counts per transcript
-    :param min_dist: minimum distance (in nucleotides) between the two Alternative Splicing Events for the pair to be tested
-    :type min_dist: int
-    :param test: test to be performed. One of ("chi2", "fisher")
-    :type test: str
-    :param sg: segment graph
-    '''
-
-    con_tab = np.zeros((2, 2), dtype=int)
-    tr_ID_tab = np.zeros((2, 2), dtype=object)
-
-    for m, n in itertools.product(range(2), range(2)):
-        tr_IDs = set(e1[m]) & set(e2[n])
-        cov_tr_IDs = [(coverage[trid], trid) for trid in tr_IDs]  # get coverage and tr_ID
-        cov_tr_IDs.sort(reverse=True)  # sort by coverage
-        tr_ID_tab[n, m] = [trid for _, trid in cov_tr_IDs]
-
-        c_mn = sum([cov for cov, _ in cov_tr_IDs])
-        con_tab[n, m] = c_mn
-
-    if test == 'chi2':
-        test_fun = chi2_contingency
-    elif test == 'fisher':
-        test_fun = fisher_exact
-
-    test_res = test_fun(con_tab+.01)  # add some small value (TODO: is this also for fisher test?)
-
-    priA_priB_trID, altA_altB_trID = tr_ID_tab[0, 0], tr_ID_tab[1, 1]
-    priA_altB_trID, altA_priB_trID = tr_ID_tab[1, 0], tr_ID_tab[0, 1]
-    priA_priB, altA_altB = con_tab[0, 0], con_tab[1, 1]
-    priA_altB, altA_priB = con_tab[1, 0], con_tab[0, 1]
-    p_value = test_res[1]
-    test_stat = test_res[0]
-    log2OR = _corrected_log2OR(con_tab)
-
-    # logOR is a measure of the effect size. coordination between the events is either positive or negative.
-
-    return_tuple = (p_value, test_stat, log2OR, priA_priB, priA_altB, altA_priB, altA_altB,
-                    priA_priB_trID, priA_altB_trID, altA_priB_trID, altA_altB_trID)
-
-    return return_tuple
-
-
-def precompute_events_dict(self, event_type=("ES", "5AS", "3AS", "IR", "ME"), min_cov=100, region=None, progress_bar=True):
-    '''
-    Precomputes the evvents_dict, i.e. a dictionary of splice bubbles. Each key is a gene and each value is the splice bubbles
-    object corresponding to that gene.
-    :param region: The region to be considered. Either a string "chr:start-end", or a tuple (chr,start,end). Start and end is optional.
-
-    '''
-
-    events_dict = {}
-
-    for g in self.iter_genes(region=region, progress_bar=progress_bar):
-        sg = g.segment_graph
-        events = sg.find_splice_bubbles(types=event_type)
-        events_dict[g.id] = [e for e in events if g.coverage.sum(axis=0)[e[0]+e[1]].sum() >= min_cov]
-
-    return events_dict
-
-
-def coordination_test(self, samples=None, test="chi2", min_dist=1, min_total=100, min_alt_fraction=.1, min_cov_pair=100,
-                      events_dict={}, event_type=("ES", "5AS", "3AS", "IR", "ME"), query=None, region=None, method="fdr_bh",
+def coordination_test(self, samples=None, test="fisher", min_dist=1, min_total=100, min_alt_fraction=.1,
+                      events_dict=None, event_type=("ES", "5AS", "3AS", "IR", "ME"), query=None, region=None, padj_method="fdr_bh",
                       progress_bar=True):
     '''
     Performs gene_coordination_test on all genes.
@@ -824,13 +789,14 @@ def coordination_test(self, samples=None, test="chi2", min_dist=1, min_total=100
     :type min_alt_frction: float
     :param min_cov_pair: the minimum total number of a pair of the joint occurrence of a pair of event for it to be reported in the result
     :type min_cov_pair: int
-    :param events_dict: a dictionary of splice bubbles. Each key is a gene and each value is the splice bubbles object corresponding to that
-     gene.
+    :param events_dict: Precomputed dictionary of alternative splicing events, to speed up analysis of several groups of samples of the same dataset.
+        Can be generated with the function _utils.precompute_events_dict.
     :param event_type:  A tuple with event types to test. Valid types are ("ES","3AS", "5AS","IR" or "ME", "TSS", "PAS").
     Default is ("ES", "5AS", "3AS", "IR", "ME")
     :param query: If provided, query string is evaluated on all genes for filtering
     :param region: The region to be considered. Either a string "chr:start-end", or a tuple (chr,start,end). Start and end is optional.
-    :param method: The multiple tet adjustment method. Any value allowed by statsmodels.stats.multitest.multipletests (std: Benjamini-Hochberg)
+    :param padj_method: The multiple test adjustment method.
+        Any value allowed by statsmodels.stats.multitest.multipletests (default: Benjamini-Hochberg)
 
     :return: a Pandas dataframe, where each column corresponds to the p_values, the statistics
     (the chi squared statistic if the chi squared test is used and the odds-ratio if the Fisher
@@ -846,26 +812,26 @@ def coordination_test(self, samples=None, test="chi2", min_dist=1, min_total=100
         _, _, groups = _check_groups(self, [samples], 1)
         samples = groups[0]
 
-    for g in self.iter_genes(region=region, progress_bar=progress_bar, query=query):
-
+    for g in self.iter_genes(region=region, query=query, progress_bar=progress_bar):
+        events = events_dict.get(g.id, []) if events_dict is not None else None
         try:
             next_test_res = g.coordination_test(test=test, samples=samples, min_dist=min_dist, min_total=min_total,
-                                                min_alt_fraction=min_alt_fraction, min_cov_pair=min_cov_pair,
-                                                events=events_dict.get(g.id), event_type=event_type)
+                                                min_alt_fraction=min_alt_fraction,
+                                                events=events, event_type=event_type)
             test_res.extend(next_test_res)
 
         except Exception:
             logger.error(f"\nError encounter on {print(g)} {g.id}   :  {g.name}.")
             raise
 
-    col_names = ("gene_id", "gene_name", "strand", "ase1_type", "ase2_type", "ase1_start", "ase1_end",
-                 "ase2_start", "ase2_end", "p_value", "stat", "log2OR", "priA_priB", "priA_altB", "altA_priB",
+    col_names = ("gene_id", "gene_name", "strand", "eventA_type", "eventB_type", "eventA_start", "evemtA_end",
+                 "eventB_start", "eventB_end", "pvalue", "stat", "log2OR", "dcPSI_AB", "dcPSI_BA", "priA_priB", "priA_altB", "altA_priB",
                  "altA_altB", "priA_priB_trID", "priA_altB_trID", "altA_priB_trID", "altA_altB_trID")
 
     res = pd.DataFrame(test_res, columns=col_names)
 
-    adj_p_value = multi.multipletests(res.p_value, method=method)[1]
+    adj_p_value = multi.multipletests(res.pvalue, method=padj_method)[1]
 
-    res.insert(10, "adj_p_value", adj_p_value)
+    res.insert(10, "padj", adj_p_value)
 
     return res
