@@ -6,6 +6,8 @@ import re
 from tqdm import tqdm
 import builtins
 import logging
+from scipy.stats import chi2_contingency, fisher_exact
+
 logger = logging.getLogger('isotools')
 
 cigar = 'MIDNSHP=XB'
@@ -128,10 +130,35 @@ def splice_identical(tr1, tr2):
     return True
 
 
+def find_orfs(seq, start_codons=["ATG"], stop_codons=['TAA', 'TAG', 'TGA'], minlen=100):
+    ''' Find all open reading frames on the forward strand of the sequence.
+    Return a 5-tuple with start and stop position, reading frame (0,1 or 2) and start and stop codon sequence
+    '''
+    orf = []
+    starts = [[], [], []]
+    stops = [[], [], []]
+    for match in re.finditer("|".join(start_codons), seq):
+        starts[match.start() % 3].append((match.start(), match.group()))
+    for match in re.finditer("|".join(stop_codons), seq):
+        stops[match.start() % 3].append((match.end(), match.group()))
+    for frame in range(3):
+        stop = (0, None)
+        for start in starts[frame]:
+            if start[0] < stop[0]:
+                continue
+            try:
+                stop = next(s for s in sorted(stops[frame]) if s[0] > start[0])
+            except StopIteration:
+                continue
+            if stop[0]-start[0] >= minlen:
+                orf.append((start[0], stop[0], frame, start[1], stop[1]))
+    return orf
+
+
 def has_overlap(r1, r2):
     "check the overlap of two intervals"
     # assuming start < end
-    if r1[1] < r2[0] or r2[1] < r1[0]:
+    if r1[1] <= r2[0] or r2[1] <= r1[0]:
         return False
     else:
         return True
@@ -240,11 +267,11 @@ def _get_overlap(exons, transcripts):
     ol = 0
     i = 0
     for e in exons:
-        while(exon_reg[i][1] < e[0]):  # no overlap, go on
+        while exon_reg[i][1] < e[0]:  # no overlap, go on
             i += 1
             if i == len(exon_reg):
                 return ol
-        while(exon_reg[i][0] < e[1]):
+        while exon_reg[i][0] < e[1]:
             i_end = min(e[1], exon_reg[i][1])
             i_start = max(e[0], exon_reg[i][0])
             ol += (i_end - i_start)
@@ -278,7 +305,7 @@ def _find_splice_sites(sj, transcripts):
     for sjs, idx in sorted(splice_junction_starts.items()):  # splice junction starts, sorted by position
         for j, tr_iter in enumerate(tr_list):
             try:
-                while(sjs > current[j][1]):
+                while sjs > current[j][1]:
                     current[j] = next(tr_iter)
                 if current[j][1] == sjs:
                     for i in idx:
@@ -292,7 +319,7 @@ def _find_splice_sites(sj, transcripts):
     for sje, idx in sorted(splice_junction_ends.items()):  # splice junction ends, sorted by position
         for j, tr_iter in enumerate(tr_list):
             try:
-                while(sje > current[j][0]):
+                while sje > current[j][0]:
                     current[j] = next(tr_iter)
                 if current[j][0] == sje:
                     for i in idx:
@@ -303,6 +330,101 @@ def _find_splice_sites(sj, transcripts):
     return sites
 
 
+def precompute_events_dict(transcriptome, event_type=("ES", "5AS", "3AS", "IR", "ME"), min_cov=100, region=None,  query=None, progress_bar=True):
+    '''
+    Precomputes the events_dict, i.e. a dictionary of splice bubbles. Each key is a gene and each value is the splice bubbles
+    object corresponding to that gene.
+    :param region: The region to be considered. Either a string "chr:start-end", or a tuple (chr,start,end). Start and end is optional.
+    '''
+
+    events_dict = {}
+
+    for g in transcriptome.iter_genes(region=region, query=query, progress_bar=progress_bar):
+        sg = g.segment_graph
+        events = [e for e in sg.find_splice_bubbles(types=event_type) if g.coverage.sum(axis=0)[e[0]+e[1]].sum() >= min_cov]
+        if events:
+            events_dict[g.id] = events
+
+    return events_dict
+
+
+def get_quantiles(pos, percentile=[.5]):
+    '''provided a list of (positions,coverage) pairs, return the median position'''
+    # percentile should be sorted, and between 0 and 1
+    total = sum(cov for _, cov in pos)
+    n = 0
+    result_list = []
+    for p, cov in sorted(pos, key=lambda x: x[0]):
+        n += cov
+        while n >= total * percentile[len(result_list)]:
+            result_list.append(p)
+            if len(result_list) == len(percentile):
+                return result_list
+    raise ValueError(f'cannot find {percentile[len(result_list)]} percentile of {pos}')
+
+
+def smooth(x, window_len=31):
+    """ smooth the data using a hanning window with requested size."""
+    # padding with mirrored
+    s = np.r_[x[window_len-1:0:-1], x, x[-2:-window_len-1:-1]]
+    # print(len(s))
+    w = np.hanning(window_len)
+    y = np.convolve(w/w.sum(), s, mode='valid')
+    return y[int(window_len/2-(window_len+1) % 2):-int(window_len/2)]
+
+
+def prepare_contingency_table(eventA, eventB, coverage):
+    '''
+    Prepare the read counts and transcript id contingency tables for two events.
+
+    Returns two 2x2 contingency tables, one with the read counts, one with the transcript events
+
+    :param eventA: First alternative splicing event obtained from .find_splice_bubbles()
+    :param eventB: Second alternative splicing event obtained from .find_splice_bubbles()
+    :param coverage: Read counts per transcript.
+    '''
+
+    con_tab = np.zeros((2, 2), dtype=int)
+    trid_tab = np.zeros((2, 2), dtype=object)
+
+    for m, n in itertools.product(range(2), range(2)):
+        trids = sorted(set(eventA[m]) & set(eventB[n]), key=coverage.__getitem__, reverse=True)
+        trid_tab[n, m] = trids
+        con_tab[n, m] = coverage[trids].sum()
+    return con_tab, trid_tab
+
+
+def pairwise_event_test(con_tab, test="fisher", pseudocount=.01):
+    '''
+    Performs an independence test on the contingency table and computes effect sizes.
+
+    :param con_tab: contingency table with the read counts
+    :param test: Test to be performed. One of ("chi2", "fisher")
+    :type test: str
+    '''
+    if test == 'chi2':
+        test_fun = chi2_contingency
+    elif test == 'fisher':
+        test_fun = fisher_exact
+    else:
+        raise (ValueError('test should be "chi2" or "fisher"'))
+
+    test_res = test_fun(con_tab+pseudocount)  # add some small value (TODO: is this also for fisher test?)
+
+    # priA_priB_trID, altA_altB_trID = tr_ID_tab[0, 0], tr_ID_tab[1, 1]
+    # priA_altB_trID, altA_priB_trID = tr_ID_tab[1, 0], tr_ID_tab[0, 1]
+    # priA_priB, altA_altB = con_tab[0, 0], con_tab[1, 1]
+    # priA_altB, altA_priB = con_tab[1, 0], con_tab[0, 1]
+    p_value = test_res[1]
+    test_stat = test_res[0]  # TODO: inconsistancy: this is the odds ratio for the fisher test and X^2 for the chisq test
+    log2OR = _corrected_log2OR(con_tab)
+    # logOR is a measure of the effect size. coordination between the events is either positive or negative.
+    dcPSI_AB, dcPSI_BA = dcPSI(con_tab)
+    # delta conditional PSI is another measure of the effect size.
+
+    return p_value, test_stat, log2OR,  dcPSI_AB, dcPSI_BA
+
+
 def _corrected_log2OR(con_tab):
     con_tab_copy = np.zeros((2, 2), dtype=float)
 
@@ -313,3 +435,40 @@ def _corrected_log2OR(con_tab):
             con_tab_copy[n, m] = con_tab[n, m]
     log2OR = np.log2((con_tab_copy[0, 0]*con_tab_copy[1, 1])) - np.log2((con_tab_copy[0, 1]*con_tab_copy[1, 0]))
     return log2OR
+
+
+def dcPSI(con_tab):
+    # delta conditional PSI of a coordinated event
+    # 1) dcPSI_AB= PSI(B | altA) - PSI(B)
+    dcPSI_AB = con_tab[1, 1]/con_tab[:, 1].sum()-con_tab[1, :].sum()/con_tab.sum(None)
+    # 2) dcPSI_BA= PSI(A | altB) - PSI(A)
+    dcPSI_BA = con_tab[1, 1]/con_tab[1, :].sum()-con_tab[:, 1].sum()/con_tab.sum(None)
+    return dcPSI_AB, dcPSI_BA
+
+
+def genomic_position(tr_pos, exons, reverse_strand):
+    tr_len = sum((e[1]-e[0]) for e in exons)
+    assert all(p <= tr_len for p in tr_pos), f'Requested positions {tr_pos} for transcript of length {tr_len}.'
+    if reverse_strand:
+        tr_pos = [tr_len-p for p in tr_pos]
+    tr_pos = sorted(set(tr_pos))
+    intron_len = 0
+    mapped_pos = []
+    i = 0
+    offset = exons[0][0]
+    for e1, e2 in pairwise(exons):
+        while offset+intron_len+tr_pos[i] < e1[1]:
+            mapped_pos.append(offset+intron_len+tr_pos[i])
+            i += 1
+            if i == len(tr_pos):
+                break
+        else:
+            intron_len += e2[0]-e1[1]
+            continue
+        break
+    else:
+        for i in range(i, len(tr_pos)):
+            mapped_pos.append(offset+intron_len+tr_pos[i])
+    if reverse_strand:  # get them back to the original
+        tr_pos = [tr_len-p for p in tr_pos]
+    return {p: mp for p, mp in zip(tr_pos, mapped_pos)}
