@@ -1,15 +1,20 @@
+from operator import itemgetter
 from intervaltree import Interval
 from collections.abc import Iterable
 from scipy.stats import chi2_contingency
-from Bio.Seq import Seq
+from scipy.signal import find_peaks
+from Bio.Seq import reverse_complement, translate
+from Bio.Data.CodonTable import TranslationError
+
+from pysam import FastaFile
 import numpy as np
 import copy
 import itertools
 from .splice_graph import SegmentGraph
 from .short_read import Coverage
 from ._transcriptome_filter import SPLICE_CATEGORY
-from ._utils import pairwise, _filter_event
-from ._transcriptome_stats import pairwise_event_test
+from ._utils import pairwise, _filter_event, find_orfs, smooth, get_quantiles, _filter_function, \
+    pairwise_event_test, prepare_contingency_table
 
 import logging
 logger = logging.getLogger('isotools')
@@ -27,13 +32,14 @@ class Gene(Interval):
         self._transcriptome = transcriptome
 
     def __str__(self):
-        return('Gene {} {}({}), {} reference transcripts, {} expressed transcripts'.format(
-            self.name, self.region, self.strand, self.n_ref_transcripts, self.n_transcripts))
+        return 'Gene {} {}({}), {} reference transcripts, {} expressed transcripts'.format(
+            self.name, self.region, self.strand, self.n_ref_transcripts, self.n_transcripts)
 
     def __repr__(self):
         return object.__repr__(self)
 
-    from ._gene_plots import sashimi_plot, gene_track, sashimi_plot_short_reads, sashimi_figure
+    from ._gene_plots import sashimi_plot, gene_track, sashimi_plot_short_reads, sashimi_figure, plot_domains
+    from .domains import add_interpro_domains
 
     def short_reads(self, idx):
         '''Returns the short read coverage profile for a short read sample.
@@ -141,7 +147,7 @@ class Gene(Interval):
             if self.strand == '+':
                 sj_seq = [ss_seq[d] + ss_seq[a] for d, a in pos]
             else:
-                sj_seq = [str(Seq(ss_seq[d] + ss_seq[a]).reverse_complement()) for d, a in pos]
+                sj_seq = [reverse_complement(ss_seq[d] + ss_seq[a]) for d, a in pos]
 
             nc = [(i, seq) for i, seq in enumerate(sj_seq) if seq != 'GTAG']
             if nc:
@@ -204,6 +210,82 @@ class Gene(Interval):
                     a_content[pos] = seq.upper().count('T') / length
             tr['downstream_A_content'] = a_content[pos]
 
+    def get_sequence(self, genome_fh, trids=None, reference=False, protein=False):
+        '''Returns the nucleotide sequence of the specified transcripts.
+
+        :param genome_fh: The path to the genome fasta file, or FastaFile handle.
+        :param trids: List of transcript ids for which the sequence are requested.
+        :param reference: Specifiy whether the sequence is fetched for reference transcripts (True)
+            or long read transcripts (False, default).
+        :param protein: Return protein sequences instead of transcript sequences.'''
+
+        trL = [(i, tr) for i, tr in enumerate(self.ref_transcripts if reference else self.transcripts) if trids is None or i in trids]
+        if not trL:
+            return {}
+        pos = (min(tr['exons'][0][0] for _, tr in trL), max(tr['exons'][-1][1] for _, tr in trL))
+        try:  # assume its a FastaFile file handle
+            seq = genome_fh.fetch(self.chrom, *pos)
+        except AttributeError:
+            genome_fn = genome_fh
+            with FastaFile(genome_fn) as genome_fh:
+                seq = genome_fh.fetch(self.chrom, *pos)
+        tr_seqs = {}
+        for i, tr in trL:
+            trseq = ''
+            for e in tr['exons']:
+                trseq += seq[e[0]-pos[0]:e[1]-pos[0]]
+            tr_seqs[i] = trseq
+
+        if self.strand == '-':
+            tr_seqs = {i: reverse_complement(ts) for i, ts in tr_seqs.items()}
+        if not protein:
+            return tr_seqs
+        prot_seqs = {}
+        for i, tr in trL:
+            orf = tr.get("CDS", tr.get("ORF"))
+            if not orf:
+                continue
+            pos = sorted(self.find_transcript_positions(i, orf[:2], reference=reference))
+            try:
+                prot_seqs[i] = translate(tr_seqs[i][pos[0]:pos[1]], cds=True)
+            except TranslationError:
+                logger.warning(f'CDS sequence of {self.id} {"reference" if reference else ""} transcript {i} cannot be translated.')
+        return prot_seqs
+
+    def add_orfs(self, genome_fh, reference=False, minlen=30, start_codons=["ATG"], stop_codons=['TAA', 'TAG', 'TGA']):
+        '''find longest ORF for each transcript and add to the transcript properties tr["ORF"]'''
+        trL = self.ref_transcripts if reference else self.transcripts
+        for (_, orfs), tr in zip(self.get_all_orf(genome_fh, reference, minlen, start_codons, stop_codons), trL):
+            if orfs:
+                tr["ORF"] = max(orfs, key=lambda x: x[2]['length'])
+
+    def get_all_orf(self, genome_fh, reference=False, minlen=30, start_codons=["ATG"], stop_codons=['TAA', 'TAG', 'TGA']):
+        ''' Predicts ORF.
+        '''
+        orf_list = []
+        trL = self.ref_transcripts if reference else self.transcripts
+        for trid, tr_seq in self.get_sequence(genome_fh, reference=reference).items():
+            tr = trL[trid]
+            tr_start = tr['exons'][0][0]
+            cum_exon_len = np.cumsum([end-start for start, end in tr['exons']])  # cummulative exon length
+            cum_intron_len = np.cumsum([0]+[end-start for (_, start), (end, _) in pairwise(tr['exons'])])  # cummulative intron length
+            orf_list.append((tr_seq, []))
+            for start, stop, frame, seq_start, seq_end in find_orfs(tr_seq, minlen=minlen):
+                if self.strand == '-':
+                    start, stop = cum_exon_len[-1]-stop, cum_exon_len[-1]-start
+                start_exon = next(i for i in range(len(cum_exon_len)) if cum_exon_len[i] >= start)
+                stop_exon = next(i for i in range(start_exon, len(cum_exon_len)) if cum_exon_len[i] >= stop)
+                genome_pos = (tr_start+start+cum_intron_len[start_exon],
+                              tr_start+stop+cum_intron_len[stop_exon])
+                dist_pas = 0  # distance of termination codon to last upstream splice site
+                if self.strand == '+' and stop_exon < len(cum_exon_len)-1:
+                    dist_pas = cum_exon_len[-2]-stop
+                if self.strand == '-' and start_exon > 0:
+                    dist_pas = start-cum_exon_len[0]
+                orf_list[-1][1].append((*genome_pos, {'start': start, 'length': stop-start,
+                                       'start_codon': seq_start, 'stop_codon': seq_end, 'NMD': dist_pas > 55}))
+        return orf_list
+
     def add_fragments(self):
         '''Checks for transcripts that are fully contained in other transcripts.
 
@@ -293,6 +375,32 @@ class Gene(Interval):
         TPM is returned as a numpy array, with samples in columns and transcript isoforms in the rows.'''
         return (self.coverage+pseudocount)/self._transcriptome.sample_table['nonchimeric_reads'].values.reshape(-1, 1)*1e6
 
+    def find_transcript_positions(self, trid, pos, reference=False):
+        '''Converts genomic positions to positions within the transcript.
+
+        :param trid: The transcript id
+        :param pos: List of sorted genomic positions, for which the transcript positions are computed.'''
+
+        tr_pos = []
+        exons = self.ref_transcripts[trid]['exons'] if reference else self.transcripts[trid]['exons']
+        e_idx = 0
+        offset = 0
+        for p in sorted(pos):
+            try:
+                while p > exons[e_idx][1]:
+                    offset += (exons[e_idx][1]-exons[e_idx][0])
+                    e_idx += 1
+
+            except IndexError:
+                for _ in range(len(pos)-len(tr_pos)):
+                    tr_pos.append(None)
+                break
+            tr_pos.append(offset+p-exons[e_idx][0] if p >= exons[e_idx][0] else None)
+        if self.strand == '-':
+            trlen = sum(end-start for start, end in exons)
+            tr_pos = [None if p is None else trlen-p for p in tr_pos]
+        return tr_pos
+
     @property
     def coverage(self):
         '''Returns the transcript coverage.
@@ -402,7 +510,11 @@ class Gene(Interval):
         '''Returns the segment graph of the LRTS transcripts for the gene'''
         if 'segment_graph' not in self.data or self.data['segment_graph'] is None:
             exons = [tr['exons'] for tr in self.transcripts]
-            self.data['segment_graph'] = SegmentGraph(exons, self.strand)
+            try:
+                self.data['segment_graph'] = SegmentGraph(exons, self.strand)
+            except Exception:
+                logger.error('Error initializing Segment Graph on %s with exons %s', self.strand, exons)
+                raise
         return self.data['segment_graph']
 
     def __copy__(self):
@@ -418,6 +530,43 @@ class Gene(Interval):
         'Returns a shallow copy of self.'
         return self.__copy__()
 
+    def filter_transcripts(self, query=None, min_coverage=None, max_coverage=None):
+        tr_filter = self._transcriptome.filter['transcript']
+        if query:
+            # used_tags={tag for tag in re.findall(r'\b\w+\b', query) if tag not in BOOL_OP}
+            query_fun, used_tags = _filter_function(query)
+            msg = 'did not find the following filter rules: {}\nvalid rules are: {}'
+            assert all(f in tr_filter for f in used_tags), msg.format(
+                ', '.join(f for f in used_tags if f not in tr_filter), ', '.join(tr_filter))
+            tr_filter_fun = {tag: _filter_function(tr_filter[tag])[0] for tag in used_tags if tag in tr_filter}
+        trids = []
+        for i, tr in enumerate(self.transcripts):
+            if min_coverage and self.coverage[:, i].sum() < min_coverage:
+                continue
+            if max_coverage and self.coverage[:, i].sum() > max_coverage:
+                continue
+            if query is None or query_fun(
+                    **{tag: f(g=self, trid=i, **tr) for tag, f in tr_filter_fun.items()}):
+                trids.append(i)
+        return trids
+
+    def filter_ref_transcripts(self, query=None):
+        tr_filter = self._transcriptome.filter['reference']
+        if query:
+            # used_tags={tag for tag in re.findall(r'\b\w+\b', query) if tag not in BOOL_OP}
+            query_fun, used_tags = _filter_function(query)
+            msg = 'did not find the following filter rules: {}\nvalid rules are: {}'
+            assert all(f in tr_filter for f in used_tags), msg.format(
+                ', '.join(f for f in used_tags if f not in tr_filter), ', '.join(tr_filter))
+            tr_filter_fun = {tag: _filter_function(tr_filter[tag])[0] for tag in used_tags if tag in tr_filter}
+        else:
+            return list(range(len(self.transcripts)))
+        trids = []
+        for i, tr in enumerate(self.ref_transcripts):
+            if query_fun(**{tag: f(g=self, trid=i, **tr) for tag, f in tr_filter_fun.items()}):
+                trids.append(i)
+        return trids
+
     def _find_splice_sites(exons, transcripts):
         '''Checks whether the splice sites of a new transcript are present in the set of transcripts.
         avoids the computation of segment graph, which provides the same functionality.
@@ -441,7 +590,7 @@ class Gene(Interval):
         return current
 
     def coordination_test(self, samples=None, test="chi2", min_dist=1, min_total=100, min_alt_fraction=.1,
-                          min_cov_pair=100, events=None, event_type=("ES", "5AS", "3AS", "IR", "ME")):
+                          events=None, event_type=("ES", "5AS", "3AS", "IR", "ME")):
         '''Performs pairwise independence test for all pairs of Alternative Splicing Events (ASEs) in a gene.
 
         For all pairs of ASEs in a gene creates a contingency table and performs an indeppendence test.
@@ -458,23 +607,18 @@ class Gene(Interval):
         :param min_dist: Minimum distance (in nucleotides) between the two
             alternative splicing events for the pair to be tested.
         :type min_dist: int
-        :param min_total: The minimum total number of reads for an event to pass the filter.
+        :param min_total: The minimum total number of reads for an event pair to pass the filter.
         :type min_total: int
-        :param min_alt_fraction: The minimum fraction of read supporting the alternative.
+        :param min_alt_fraction: The minimum fraction of reads supporting the minor alternative of the two events.
         :type min_alt_fraction: float
-        :param min_cov_pair: the minimum total number of a pair of the joint occurrence of a pair of event for it to be reported in the result
-        :type min_cov_pair: int
-        :param events: a splice_bubble object
+        :param events: To speed up testing on different groups of the same transcriptome objects, events can be precomputed
+            with the isotools._utils.precompute_events_dict function.
         :param event_type:  A tuple with event types to test. Valid types are
         ("ES", "3AS", "5AS", "IR", "ME", "TSS", "PAS"). Default is ("ES", "5AS", "3AS", "IR", "ME").
         Not used if the event parameter is already given.
-        :return: A list of tuples (gene_id, gene_name, strand, ASE1_type, ASE2_type,
-        ASE1_start, ASE1_end, ASE2_start, ASE2_end, priA_priB, priA_altB, altA_priB, altA_altB),
-        where each entrance in the tuple corresponds to the p_value, the statistic, the gene name,
-        the type of the first ASE, the type of the second ASE, the starting coordinate of the first ASE,
-        the ending coordinate of the first ASE, the starting coordinate of the second ASE,
-        the ending coordinate of the second ASE, the four entries of the contingency table for each test performed (a column for each),
-        and the four lists of the transcript IDs sorted by coverage (a column for each list).
+        :return: A list of tuples with the test results: (gene_id, gene_name, strand, eventA_type, eventB_type,
+            eventA_start, eventA_end, eventB_start, eventB_end, p_value, test_stat, log2OR,  dcPSI_AB, dcPSI_BA,
+            priA_priB, priA_altB, altA_priB, altA_altB,  priA_priB_trids, priA_altB_trids, altA_priB_trids, altA_altB_trids).
         '''
 
         if samples is None:
@@ -496,7 +640,11 @@ class Gene(Interval):
 
         events = [e for e in events if _filter_event(cov, e, min_total=min_total,
                                                      min_alt_fraction=min_alt_fraction)]
-
+        # make sure its sorted (according to gene strand)
+        if self.strand == '+':
+            events.sort(key=itemgetter(2, 3), reverse=False)  # sort by starting node
+        else:
+            events.sort(key=itemgetter(3, 2), reverse=True)  # reverse sort by end node
         test_res = []
 
         for i, j in itertools.combinations(range(len(events)), 2):
@@ -505,16 +653,20 @@ class Gene(Interval):
             if (events[i][4], events[j][4]) == ("TSS", "TSS") or (events[i][4], events[j][4]) == ("PAS", "PAS"):
                 continue
 
-            attr = pairwise_event_test(events[i], events[j], cov, test=test)  # append to test result
+            con_tab, tr_ID_tab = prepare_contingency_table(events[i], events[j], cov)
 
-            if sum(attr[3:7]) < min_cov_pair:  # check that the joint occurrence of the two events passes the threshold
+            if con_tab.sum(None) < min_total:  # check that the joint occurrence of the two events passes the threshold
                 continue
+            if min(con_tab.sum(1).min(), con_tab.sum(0).min())/con_tab.sum(None) < min_alt_fraction:
+                continue
+            test_result = pairwise_event_test(con_tab, test=test)  # append to test result
 
             coordinate1 = sg._get_event_coordinate(events[i])
             coordinate2 = sg._get_event_coordinate(events[j])
 
-            attr = (self.id, self.name, self.strand, events[i][4], events[j][4], coordinate1[0],
-                    coordinate1[1], coordinate2[0], coordinate2[1]) + attr
+            attr = (self.id, self.name, self.strand, events[i][4], events[j][4]) + \
+                coordinate1 + coordinate2 + test_result + \
+                tuple(con_tab.flatten()) + tuple(tr_ID_tab.flatten())
 
             # events[i][4] is the events[i] type
             # coordinate1[0] is the starting coordinate of event 1
@@ -549,8 +701,7 @@ class Gene(Interval):
             return np.nan, np.nan, []
         # if there are more than 'numIsoforms' isoforms of the gene, all additional least expressed get summarized.
         if cov.shape[0] > n_isoforms:
-            idx = np.argpartition(-cov.sum(1), n_isoforms)
-
+            idx = np.argpartition(-cov.sum(1), n_isoforms)  # take the n_isoforms most expressed isoforms (random order)
             additional = cov[idx[n_isoforms:]].sum(0)
             cov = cov[idx[:n_isoforms]]
             cov[n_isoforms-1] += additional
@@ -562,7 +713,7 @@ class Gene(Interval):
         try:
             _, pval, _, _ = chi2_contingency(cov)
         except ValueError:
-            logger.debug(f'chi2_contingency({cov})')
+            logger.error(f'chi2_contingency({cov})')
             raise
         iso_frac = cov/cov.sum(0)
         deltaPI = iso_frac[..., 0]-iso_frac[..., 1]
@@ -576,6 +727,133 @@ class Gene(Interval):
         else:
             return pval, deltaPI_neg, idx[neg_idx]
 
+    def _unify_ends(self, smooth_window=31, rel_prominence=1, search_range=(.1, .9)):
+        ''' Find common TSS/PAS for tanscripts of the gene'''
+        if not self.transcripts:
+            # nothing to do here
+            return
+        assert 0 <= search_range[0] <= .5 <= search_range[1] <= 1
+        # get gene tss/pas profiles
+        tss = {}
+        pas = {}
+        for tr in self.transcripts:
+            for sa in tr['TSS']:
+                for pos, c in tr['TSS'][sa].items():
+                    tss[pos] = tss.get(pos, 0)+c
+            for sa in tr['PAS']:
+                for pos, c in tr['PAS'][sa].items():
+                    pas[pos] = pas.get(pos, 0)+c
+
+        tss_pos = [min(tss), max(tss)]
+        if tss_pos[1]-tss_pos[0] < smooth_window:
+            tss_pos[0] -= (smooth_window + tss_pos[0]-tss_pos[1] - 1)
+        pas_pos = [min(pas), max(pas)]
+        if pas_pos[1]-pas_pos[0] < smooth_window:
+            pas_pos[0] -= (smooth_window + pas_pos[0]-pas_pos[1] - 1)
+        tss = [tss.get(pos, 0) for pos in range(tss_pos[0], tss_pos[1]+1)]
+        pas = [pas.get(pos, 0) for pos in range(pas_pos[0], pas_pos[1]+1)]
+        # smooth profiles and finde maxima
+        tss_smooth = smooth(np.array(tss), smooth_window)
+        pas_smooth = smooth(np.array(pas), smooth_window)
+        # at least half of smooth_window reads required to call a peak
+        # minimal distance between peaks is > ~ smooth_window
+        # rel_prominence=1 -> smaller peak must have twice the hight of valley to call two peaks
+        tss_peaks, _ = find_peaks(np.log2(tss_smooth+1), prominence=(rel_prominence, None))
+        tss_peak_pos = tss_peaks+tss_pos[0]
+        pas_peaks, _ = find_peaks(np.log2(pas_smooth+1), prominence=(rel_prominence, None))
+        pas_peak_pos = pas_peaks+pas_pos[0]
+
+        # find transcripts with common first/last splice site
+        starts = {}
+        ends = {}
+        for trid, tr in enumerate(self.transcripts):
+            starts.setdefault(tr['exons'][0][1], []).append(trid)
+            ends.setdefault(tr['exons'][-1][0], []).append(trid)
+        if self.strand == '-':
+            starts, ends = ends, starts
+        # for each site, find consistant "peaks" TSS/PAS
+        # if none found use median of all read starts
+        for pos, tr_ids in starts.items():
+            profile = {}
+            for trid in tr_ids:
+                for sa_tss in self.transcripts[trid]['TSS'].values():
+                    for pos, c in sa_tss.items():
+                        profile[pos] = profile.get(pos, 0)+c
+            quantiles = get_quantiles(sorted(profile.items()), [search_range[0], .5, search_range[1]])
+            # one/ several peaks within base range? -> quantify by next read_start
+            # else use median
+            ol_peaks = [p for p in tss_peak_pos if quantiles[0] < p <= quantiles[-1]]
+            if not ol_peaks:
+                ol_peaks = [quantiles[1]]
+            for trid in tr_ids:
+                tr = self.transcripts[trid]
+                tr['TSS_unified'] = {}
+                for sa, sa_tss in tr['TSS'].items():
+                    tss_unified = {}
+                    for pos, c in sa_tss.items():
+                        next_peak = min((p for p in ol_peaks if p < tr['exons'][0][1]),
+                                        default=pos, key=lambda x: abs(x-pos))
+                        tss_unified[next_peak] = tss_unified.get(next_peak, 0)+c
+                    tr['TSS_unified'][sa] = tss_unified
+        # same for PAS
+        for pos, tr_ids in ends.items():
+            profile = {}
+            for trid in tr_ids:
+                for sa_pas in self.transcripts[trid]['PAS'].values():
+                    for pos, c in sa_pas.items():
+                        profile[pos] = profile.get(pos, 0)+c
+            quantiles = get_quantiles(sorted(profile.items()), [search_range[0], .5, search_range[1]])
+            # one/ several peaks within base range? -> quantify by next read_start
+            # else use median
+            ol_peaks = [p for p in pas_peak_pos if quantiles[0] < p <= quantiles[-1]]
+            if not ol_peaks:
+                ol_peaks = [quantiles[1]]
+            for trid in tr_ids:
+                tr = self.transcripts[trid]
+                tr['PAS_unified'] = {}
+                for sa, sa_pas in tr['PAS'].items():
+                    pas_unified = {}
+                    for pos, c in sa_pas.items():
+                        next_peak = min((p for p in ol_peaks if p > tr['exons'][-1][0]),
+                                        default=pos, key=lambda x: abs(x-pos))
+                        pas_unified[next_peak] = pas_unified.get(next_peak, 0)+c
+                    tr['PAS_unified'][sa] = pas_unified
+        for tr in self.transcripts:
+            # find the most common tss/pas per transcript, and set the exon boundaries
+            sum_tss = {}
+            sum_pas = {}
+            start = end = max_tss = max_pas = 0
+            for sa_tss in tr['TSS_unified'].values():
+                for pos, cov in sa_tss.items():
+                    sum_tss[pos] = sum_tss.get(pos, 0)+cov
+            for pos, cov in sum_tss.items():
+                if cov > max_tss:
+                    max_tss = cov
+                    start = pos
+            for sa_pas in tr['PAS_unified'].values():
+                for pos, cov in sa_pas.items():
+                    sum_pas[pos] = sum_pas.get(pos, 0)+cov
+            for pos, cov in sum_pas.items():
+                if cov > max_pas:
+                    max_pas = cov
+                    end = pos
+            if self.strand == '-':
+                start, end = end, start
+            if start > end:  # for monoexons this may happen in rare situations
+                assert len(tr['exons']) == 1
+                tr['TSS_unified'] = None
+                tr['PAS_unified'] = None
+            else:
+                try:
+                    # issues if the new exon start is behind the exon end
+                    assert start < tr['exons'][0][1] or len(tr['exons']) == 1, 'error unifying %s: %s>=%s' % (tr["exons"], start, tr['exons'][0][1])
+                    tr['exons'][0][0] = start
+                    assert end > tr['exons'][-1][0] or len(tr['exons']) == 1, 'error unifying %s: %s<=%s' % (tr["exons"], end, tr['exons'][-1][0])
+                    tr['exons'][-1][1] = end
+                except AssertionError:
+                    logger.error('%s TSS= %s, PAS=%s -> TSS_unified= %s, PAS_unified=%s', self, tr['TSS'], tr['PAS'],  tr['TSS_unified'], tr['PAS_unified'])
+                    raise
+
 
 def _coding_len(exons, cds):
     coding_len = [0, 0, 0]
